@@ -62,6 +62,10 @@ val process_start_tag:
   table -> string -> (string * string) list -> 
     table * QName.t * (Label.t * Utf8.t) list
 
+val process_start_tag_subst:
+  table -> string -> (string * string) list ->
+  Uri.t Hashtbl.Make(Uri).t -> table * QName.t * (Label.t * Utf8.t) list 
+
 val map_tag: table -> Utf8.t -> QName.t
 val map_attr: table -> Utf8.t -> QName.t
 val map_prefix: table -> Utf8.t -> Uri.t
