@@ -15,6 +15,8 @@ let rhs_loc = dummy
 let input_name = ref ""
 let input_lexbuf = ref None
 let get_pos_info = dummy
+let print_error_cur_file = dummy
+let print_error = dummy
 let print = dummy
 let print_warning = dummy
 let prerr_warning = dummy
