@@ -5,6 +5,8 @@
 (**************************************************************************)
 
 val load_xml: ?ns:bool -> string -> Value.t
+val load_xml_subst: ?ns:bool -> string -> 
+  (Ns.Uri.t * Ns.Uri.t) list -> Value.t
 val load_html: string -> Value.t
 
 
