@@ -61,6 +61,7 @@ let write_data_string ~to_enc buf s =
 open Value
 open Ident
 module U = Encodings.Utf8
+module H = Hashtbl.Make(Ns.Uri)
 
 let exn_print_xml = CDuceExn (Pair (
 				Atom (Atoms.V.mk_ascii "Invalid_argument"),
@@ -100,7 +101,7 @@ and schema_values ~wds ~wcs v =
       schema_values ~wds ~wcs tl
   | _ -> raise exn_print_xml
 
-let to_buf ~utf8 buffer ns_table v = 
+let to_buf ~utf8 buffer ns_table v subst = 
   let to_enc = if utf8 then `Enc_utf8 else `Enc_iso88591 in
 
   let printer = Ns.Printer.printer ns_table in
@@ -189,15 +190,29 @@ let to_buf ~utf8 buffer ns_table v =
       | Atom a when a = Sequence.nil_atom -> ()
       | v -> schema_value ~wds ~wcs v
   in
+  let uri_subst prefixes replace =
+    let h = H.create 10 in
+    List.iter (fun (k,v) -> H.replace h k v) replace;
+    List.map (fun (pr,ns) -> if H.mem h ns then (pr,H.find h ns) else
+	(pr,ns)) prefixes
+  in
   document_start ();
-  print_elt (Ns.Printer.prefixes printer) v
+  match subst with
+      [] -> print_elt (Ns.Printer.prefixes printer) v
+    | _ -> print_elt (uri_subst (Ns.Printer.prefixes printer) subst) v
 
 let print_xml ~utf8 ns_table s =
   let buf = Buffer.create 32 in
-  to_buf ~utf8 (Buffer.add_string buf) ns_table s;
+  to_buf ~utf8 (Buffer.add_string buf) ns_table s [];
+  let s = Buffer.contents buf in
+  if utf8 then string_utf8 (U.mk s) else string_latin1 s
+
+let print_xml_subst ~utf8 ns_table s subst =
+  let buf = Buffer.create 32 in
+  to_buf ~utf8 (Buffer.add_string buf) ns_table s subst;
   let s = Buffer.contents buf in
   if utf8 then string_utf8 (U.mk s) else string_latin1 s
  
 let dump_xml  ~utf8 ns_table s =
-  to_buf ~utf8 print_string ns_table s;
+  to_buf ~utf8 print_string ns_table s [];
   Value.nil
