@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005,2006,2007 (see LICENSE for details)  *)
+(*  Copyright CNRS,INRIA, 2003-2009 (see LICENSE for details)             *)
 (**************************************************************************)
 
 let debug = false
@@ -273,13 +273,17 @@ struct
 end
 
   (** {2 Simple type validation} *)
-
-let rec validate_simple_type def s = match def with
+let rec validate_simple_type def s =
+ match def with
   | { st_name = Some name } when Schema_builtin.is name ->
-      (try Schema_builtin.validate (Schema_builtin.get name) s
-       with Schema_builtin.Error name ->
-	 error (sprintf "%s isn't a valid %s"
-		  (Utf8.to_string s) name))
+    let value = (match def with 
+	{ st_variety = Atomic st; st_facets = facets } ->
+	  normalize_white_space (fst facets.whiteSpace) s
+      | _ -> s) in
+    (try Schema_builtin.validate (Schema_builtin.get name) value
+     with Schema_builtin.Error name ->
+       error (sprintf "%s isn't a valid %s"
+		(Utf8.to_string value) name))
   | { st_variety = Atomic st; st_facets = facets } ->
       let literal = normalize_white_space (fst facets.whiteSpace) s in
       let value = validate_simple_type st literal in
