@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005,2006,2007 (see LICENSE for details)  *)
+(*  Copyright CNRS,INRIA, 2003-2009 (see LICENSE for details)             *)
 (**************************************************************************)
 
 open Ident
@@ -13,7 +13,7 @@ let () =
   Stats.register Stats.Summary
     (fun ppf -> Format.fprintf ppf "Allocated type nodes:%i@\n" !count)
 
-(*
+(* 
 To be sure not to use generic comparison ...
 *)
 let (=) : int -> int -> bool = (==)
@@ -31,6 +31,30 @@ type const =
   | Record of const label_map
   | String of U.uindex * U.uindex * U.t * const
 
+type service_params =
+  | TProd of service_params * service_params
+  | TOption of service_params 
+  | TList of string * service_params
+  | TSet of service_params
+  | TSum of service_params * service_params
+  | TString of string
+  | TInt of string
+  | TInt32 of string
+  | TInt64 of string
+  | TFloat of string 
+  | TBool of string
+  | TFile of string
+      (* | TUserType of string * (string -> 'a) * ('a -> string) *)
+  | TCoord of string 
+  | TCoordv of service_params * string
+  | TESuffix of string 
+  | TESuffixs of string
+      (*  | TESuffixu of (string * (string -> 'a) * ('a -> string)) *)
+  | TSuffix of (bool * service_params)
+  | TUnit 
+  | TAny
+  | TConst of string;;
+
 module Const = struct
   type t = const
 
@@ -1654,7 +1678,8 @@ struct
     if (pri >= 2) && (List.length def >= 2) 
     then Format.fprintf ppf "@[(%a)@]" aux def
     else aux ppf def
-  and do_print pri ppf = function
+  and do_print pri ppf = 
+    function
     | Neg { def = [] } -> Format.fprintf ppf "Any"
     | Neg t -> Format.fprintf ppf "Any \\ (@[%a@])" (do_print_slot 0) t
     | Abs t -> Format.fprintf ppf "?(@[%a@])" (do_print_slot 0) t
@@ -1754,7 +1779,7 @@ struct
 
 
   let get_name = function
-    | { state = `Named n } -> n
+      | { state = `Named n } -> n
     | _ -> assert false
 
   let print ppf t =
@@ -1803,6 +1828,188 @@ struct
     
 end
 
+module Service =
+struct
+  type service_attributs = {
+    mutable const : bool;
+    mutable end_suffix : bool;
+    mutable file : bool
+  }
+  let prepare t = 
+    let t = Print.uniq t in
+    let t = Print.prepare t in 
+    Print.assign_name t;
+    t;;
+
+  let trace msg = (* output_string stderr (msg ^ "\n"); 
+		     flush stderr *) ();;
+
+  let print_to_string f =
+    let b = Buffer.create 1024 in
+    let ppf = Format.formatter_of_buffer b in
+    f ppf;
+    Format.pp_print_flush ppf ();
+    Buffer.contents b
+
+  let get_gname (cu,n) =
+    Ns.QName.to_string n;;
+
+  let get_gtype t =
+    get_gname t;;
+
+(* from ns:atom, returns :atom. *)
+  let strip_namespace tagname =
+    let len = String.length tagname in
+    let cur = ref len in
+    for i = 0 to (len - 1) do
+      let c = String.get tagname i in
+      match c with
+	| ':' -> cur := i
+	| _ -> ()
+    done;
+    if !cur = len then
+      tagname
+    else
+      String.sub tagname !cur (len - !cur);;
+
+  let convert_gtype t name =
+    match t with 
+      | "Int" -> TInt(name)
+      | "String" -> TString(name)
+      | "Float" -> TFloat(name)
+      | "Bool" -> TBool(name)
+      | _ -> assert false;;
+
+  let rec convert (s : Print.nd) name = 
+    trace ("debug:convert: " ^ name);
+    match s.Print.state with 
+      | `Named n -> trace ("debug:convert " ^ (U.to_string n)) ; 
+	  convert_real name s.Print.def
+      | `GlobalName n -> 
+	  let t = get_gtype n in
+	  trace("debug:convert:globalname: " ^ t);
+	  (match t with 
+	    | "Int" | "String" | "Float" | "Bool" 
+		-> convert_gtype t name
+	    | _ -> convert_real name s.Print.def)
+      | _ -> convert_real name s.Print.def
+  and convert_real name def = 
+    let rec aux = function
+      | [] -> trace ("debug:convert_real:" ^ name); assert false
+      | [ h ] -> convert_expr name h
+      | h :: t -> assert false
+    in 
+    aux def
+  and convert_expr name = function 
+    | Print.Neg { Print.def = [] } -> assert false
+    | Print.Neg t -> convert t name
+    | Print.Abs t -> convert t name
+    | Print.Name n -> assert false
+    | Print.Char c -> assert false
+    | Print.Regexp r -> convert_regexp name r
+    | Print.Xml (tag,attr,t) -> 
+      let flags = { const = false; end_suffix = false; file = false } in
+	(convert_attrs flags attr;
+	let tagname = convert_tag tag in
+	let tagname = strip_namespace tagname in
+	match tagname with
+	  | ":suffix" -> TSuffix(true,(convert t tagname))
+	  | ":any" -> TAny
+	  | _ -> (if flags.const then
+	      TConst tagname
+	    else
+	      if flags.end_suffix then
+		TESuffix tagname
+	      else
+		if flags.file then
+		  TFile tagname
+		else
+		  convert t tagname)
+	)
+    | _ -> assert false
+  and convert_regexp name = function
+    | Pretty.Seq (r1,r2) -> 
+	TProd((convert_regexp name r1),(convert_regexp name r2))
+    | Pretty.Alt (r,Pretty.Epsilon) | Pretty.Alt (Pretty.Epsilon,r) -> 
+	  TOption((convert_regexp name r))
+    | Pretty.Alt (r1,r2) -> 
+	TSum((convert_regexp name r1), (convert_regexp name r2))
+    | Pretty.Star r -> TList(name,(convert_regexp name r))
+    | Pretty.Plus r -> TList(name,(convert_regexp name r))
+    | Pretty.Trans t -> convert t name
+    | Pretty.Epsilon | Pretty.Empty -> TUnit
+  and convert_tag =
+    function 
+      | `Tag s -> print_to_string s
+      | `Type t -> assert false
+  and convert_attrs flags = 
+    trace "convert_attrs";
+    function
+      | { Print.state = `Marked|`Expand|`None; 
+	def = [ Print.Record (r,some,none) ] } -> 
+	  convert_record flags (r,some,none)
+      | { Print.state = `Named n; 
+	def = [ Print.Record (r,some,none) ] } -> 
+	  trace ("debug:convert_attrs:Named " 
+	  ^ (U.to_string n)); 
+	  convert_record flags (r,some,none)
+	    (*	  convert_real name s.Print.def *)
+	    (*      | `GlobalName n -> get_gtype n name *)
+      | _ -> trace "convert_attrs:_"; ()
+  and convert_record flags (r,some,none) =
+    List.iter
+      (fun (l,(o,t)) ->
+	(*	let opt = if o then "?" else "" in *)
+	let attr_label = Label.string_of_attr l in
+	trace ("convert_record:" ^ attr_label);
+	match attr_label with
+	  | "const" -> flags.const <- true
+	  | "end_suffix" -> flags.end_suffix <- true
+	  | "file" -> flags.file <- true
+	  | _ -> output_string stderr 
+	      ("Bad attribute name:" ^ attr_label ^ "\n")
+	  (*	  Label.print_attr l opt (do_print_slot 0) t *)
+      ) (LabelMap.get r);
+    if not none then output_string stderr " (+others)";
+    if some then output_string stderr " ..";;
+
+  let clear () =
+    Print.count_name := 0; 
+    Print.to_print := [];
+    DescrHash.clear Print.memo;;
+
+  let to_service_params t = 
+    Print.unregister_global t;
+    let s = prepare t in
+    let ret = convert s "" in
+    clear ();
+    ret;;
+
+  let to_string t =
+    let bool_to_string = function
+      | true -> "true"
+      | false -> "false" 
+    in
+    let rec aux = function 
+      | TInt n -> "TInt(" ^ n ^ ")"
+      | TFloat n -> "TFloat(" ^ n ^ ")"
+      | TBool n -> "TBool(" ^ n ^ ")"
+      | TString n -> "TString(" ^ n ^ ")"
+      | TConst n -> "TConst(" ^ n ^ ")"
+      | TProd (e1, e2) -> "TProd(" ^ (aux e1) ^ "," ^ (aux e2) ^ ")"
+      | TOption e -> "TOption(" ^ (aux e) ^ ")"
+      | TSet e -> "TSet(" ^ (aux e) ^ ")"
+      | TList (n,e) -> "TList(" ^ n ^ "," ^ (aux e) ^ ")"
+      | TUnit -> "TUnit()"
+      | TSum (e1, e2) -> "TSum(" ^ (aux e1) ^ "," ^ (aux e2) ^ ")"
+      | TSuffix (b,e) -> "TSuffix(" ^ (bool_to_string b) ^ "," ^ (aux e) ^ ")"
+      | TESuffix n -> "TESuffix(" ^ n ^ ")"
+      | TFile n -> "TFile(" ^ n ^ ")"
+      | TAny -> "TAny"
+      | _ -> " unknown "
+    in aux t;;
+end
+
 module Positive =
 struct
   type rhs = [ `Type of descr | `Cup of v list | `Times of v * v | `Xml of v * v ]
