@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005,2006,2007 (see LICENSE for details)  *)
+(*  Copyright CNRS,INRIA, 2003-2009 (see LICENSE for details)             *)
 (**************************************************************************)
 
 open Ident
@@ -15,6 +15,29 @@ type const =
   | Record of const label_map
   | String of U.uindex * U.uindex * U.t * const
 
+type service_params =
+  | TProd of service_params * service_params
+  | TOption of service_params 
+  | TList of string * service_params
+  | TSet of service_params
+  | TSum of service_params * service_params
+  | TString of string
+  | TInt of string
+  | TInt32 of string
+  | TInt64 of string
+  | TFloat of string 
+  | TBool of string
+  | TFile of string
+      (* | TUserType of string * (string -> 'a) * ('a -> string) *)
+  | TCoord of string 
+  | TCoordv of service_params * string
+  | TESuffix of string 
+  | TESuffixs of string
+      (*  | TESuffixu of (string * (string -> 'a) * ('a -> string)) *)
+  | TSuffix of (bool * service_params)
+  | TUnit 
+  | TAny
+  | TConst of string;;
 
 module Const: Custom.T with type t = const
 
@@ -289,6 +312,12 @@ sig
   val to_string: t -> string
 end
 
+module Service :
+sig
+  val to_service_params: t -> service_params
+  val to_string: service_params -> string
+end
+
 module Witness: sig
   type witness
   val print_witness: Format.formatter -> witness -> unit
